@@ -41,13 +41,14 @@
 #include "ho_objmap.h"
 #include "ho_segment.h"
 #include "ho_font.h"
+#include "ho_linguistics.h"
 
-#include "fonts/ho_recognize_font_1.h"
-#include "fonts/ho_recognize_font_2.h"
-#include "fonts/ho_recognize_font_3.h"
-#include "fonts/ho_recognize_font_4.h"
-#include "fonts/ho_recognize_font_5.h"
-#include "fonts/ho_recognize_font_6.h"
+#include "ho_recognize_font_1.h"
+#include "ho_recognize_font_2.h"
+#include "ho_recognize_font_3.h"
+#include "ho_recognize_font_4.h"
+#include "ho_recognize_font_5.h"
+#include "ho_recognize_font_6.h"
 
 #include "ho_recognize.h"
 
